package shop.itbug.bluetooth_scanner_gun_example

import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothClass
import android.bluetooth.BluetoothDevice
import android.os.Handler
import android.view.InputDevice
import android.view.KeyEvent


/**
 * 扫码枪事件解析类 by chen
 */
class ScanGunKeyEventHelper(private var mOnScanSuccessListener: OnScanSuccessListener?) {
    private val mStringBufferResult //扫码内容
            : StringBuffer
    private var mCaps //大小写区分
            = false
    private val mHandler: Handler
    private val mBluetoothAdapter: BluetoothAdapter?
    private val mScanningFishedRunnable: Runnable
    private var mDeviceName: String? = null

    /**
     * 返回扫码成功后的结果
     */
    private fun performScanSuccess() {
        val barcode = mStringBufferResult.toString()
        if (mOnScanSuccessListener != null) mOnScanSuccessListener!!.onScanSuccess(barcode)
        mStringBufferResult.setLength(0)
    }

    /**
     * 扫码枪事件解析
     * @param event
     */
    fun analysisKeyEvent(event: KeyEvent) {
        val keyCode: Int = event.getKeyCode()
        //字母大小写判断
        checkLetterStatus(event)
        if (event.getAction() === KeyEvent.ACTION_DOWN) {
            val aChar = getInputCode(event)
            if (aChar.toInt() != 0) {
                mStringBufferResult.append(aChar)
            }
            if (keyCode == KeyEvent.KEYCODE_ENTER) {
                //若为回车键，直接返回
                mHandler.removeCallbacks(mScanningFishedRunnable)
                mHandler.post(mScanningFishedRunnable)
            } else {
                //延迟post，若500ms内，有其他事件
                mHandler.removeCallbacks(mScanningFishedRunnable)
                mHandler.postDelayed(mScanningFishedRunnable, MESSAGE_DELAY)
            }
        }
    }

    //检查shift键
    private fun checkLetterStatus(event: KeyEvent) {
        val keyCode: Int = event.getKeyCode()
        if (keyCode == KeyEvent.KEYCODE_SHIFT_RIGHT || keyCode == KeyEvent.KEYCODE_SHIFT_LEFT) {
            mCaps = if (event.getAction() === KeyEvent.ACTION_DOWN) {
                //按着shift键，表示大写
                true
            } else {
                //松开shift键，表示小写
                false
            }
        }
    }

    //获取扫描内容
    private fun getInputCode(event: KeyEvent): Char {
        val keyCode = event.keyCode

        val aChar: Char

        aChar = if (keyCode >= KeyEvent.KEYCODE_A && keyCode <= KeyEvent.KEYCODE_Z) {
            //字母
            ((if (mCaps) 'A' else 'a').toInt() + keyCode - KeyEvent.KEYCODE_A).toChar()
        } else if (keyCode >= KeyEvent.KEYCODE_0 && keyCode <= KeyEvent.KEYCODE_9) {
            //数字
            ('0'.toInt() + keyCode - KeyEvent.KEYCODE_0).toChar()
        } else {
            //其他符号
            when (keyCode) {
                KeyEvent.KEYCODE_PERIOD -> '.'
                KeyEvent.KEYCODE_MINUS -> if (mCaps) '_' else '-'
                KeyEvent.KEYCODE_SLASH -> '/'
                KeyEvent.KEYCODE_BACKSLASH -> if (mCaps) '|' else '\\'
                else -> 0.toChar()
            }
        }

        return aChar
    }

    interface OnScanSuccessListener {
        fun onScanSuccess(barcode: String?)
    }

    fun onDestroy() {
        mHandler.removeCallbacks(mScanningFishedRunnable)
        mOnScanSuccessListener = null
    }
    //部分手机如三星，无法使用该方法
    // private void hasScanGun() {
    // Configuration cfg = getResources().getConfiguration();
    // return cfg.keyboard != Configuration.KEYBOARD_NOKEYS;
    // }
    /**
     * 扫描枪是否连接
     * @return
     */
    fun hasScanGun(): Boolean {
        if (mBluetoothAdapter == null) {
            return false
        }
        val blueDevices = mBluetoothAdapter.bondedDevices
        if (blueDevices == null || blueDevices.size <= 0) {
            return false
        }
        val iterator: Iterator<BluetoothDevice> = blueDevices.iterator()
        while (iterator.hasNext()) {
            val bluetoothDevice = iterator.next()
            if (bluetoothDevice.bluetoothClass.majorDeviceClass == BluetoothClass.Device.Major.PERIPHERAL) {
                mDeviceName = bluetoothDevice.name
                return isInputDeviceExist(mDeviceName)
            }
        }
        return false
    }

    /**
     * 输入设备是否存在
     * @param deviceName
     * @return
     */
    private fun isInputDeviceExist(deviceName: String?): Boolean {
        val deviceIds = InputDevice.getDeviceIds()
        for (id in deviceIds) {
            if (InputDevice.getDevice(id).name == deviceName) {
                return true
            }
        }
        return false
    }

    /**
     * 是否为扫码枪事件(部分机型KeyEvent获取的名字错误)
     * @param event
     * @return
     */
    @Deprecated("")
    fun isScanGunEvent(event: KeyEvent): Boolean {
        return event.getDevice().getName().equals(mDeviceName)
    }

    companion object {
        private const val MESSAGE_DELAY: Long = 500 //延迟500ms，判断扫码是否完成。
    }

    init {
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
        mStringBufferResult = StringBuffer()
        mHandler = Handler()
        mScanningFishedRunnable = Runnable { performScanSuccess() }
    }
}